@@ -2,24 +2,26 @@
 sentry.utils.javascript
 ~~~~~~~~~~~~~~~~~~~~~~~
 
-:copyright: (c) 2010-2013 by the Sentry Team, see AUTHORS for more details.
+:copyright: (c) 2010-2014 by the Sentry Team, see AUTHORS for more details.
 :license: BSD, see LICENSE for more details.
 """
-import time
+from __future__ import absolute_import
 
 from collections import defaultdict
-
+from datetime import timedelta
 from django.core.urlresolvers import reverse
+from django.utils import timezone
 from django.utils.html import escape
 
-from sentry.app import env
-from sentry.constants import STATUS_RESOLVED, TAG_LABELS
+from sentry.app import env, tsdb
+from sentry.constants import TAG_LABELS
 from sentry.models import (
-    Group, GroupBookmark, GroupTagKey, GroupSeen, ProjectOption
+    Group, GroupBookmark, GroupTagKey, GroupSeen, GroupStatus, ProjectOption
 )
 from sentry.templatetags.sentry_plugins import get_tags
 from sentry.utils import json
 from sentry.utils.db import attach_foreignkey
+from sentry.utils.http import absolute_uri
 
 
 transformers = {}
@@ -89,10 +91,14 @@ class GroupTransformer(Transformer):
             seen_groups = {}
 
         if objects:
-            historical_data = Group.objects.get_chart_data_for_group(
-                instances=objects,
-                max_days=1,
-                key='group',
+            end = timezone.now()
+            start = end - timedelta(days=1)
+
+            historical_data = tsdb.get_range(
+                model=tsdb.models.group,
+                keys=[g.id for g in objects],
+                start=start,
+                end=end,
             )
         else:
             historical_data = {}
@@ -140,6 +146,19 @@ class GroupTransformer(Transformer):
         return dt.isoformat()
 
     def transform(self, obj, request=None):
+        status = obj.get_status()
+        if status == GroupStatus.RESOLVED:
+            status_label = 'resolved'
+        elif status == GroupStatus.MUTED:
+            status_label = 'muted'
+        else:
+            status_label = 'unresolved'
+
+        version = obj.last_seen
+        if obj.resolved_at:
+            version = max(obj.resolved_at, obj.last_seen)
+        version = int(version.strftime('%s'))
+
         d = {
             'id': str(obj.id),
             'count': str(obj.times_seen),
@@ -148,20 +167,20 @@ class GroupTransformer(Transformer):
             'level': obj.level,
             'levelName': escape(obj.get_level_display()),
             'logger': escape(obj.logger),
-            'permalink': reverse('sentry-group', args=[obj.team.slug, obj.project.slug, obj.id]),
-            'versions': list(obj.get_version() or []),
+            'permalink': absolute_uri(reverse('sentry-group', args=[obj.organization.slug, obj.project.slug, obj.id])),
             'firstSeen': self.localize_datetime(obj.first_seen, request=request),
             'lastSeen': self.localize_datetime(obj.last_seen, request=request),
             'timeSpent': obj.avg_time_spent,
             'canResolve': request and request.user.is_authenticated(),
-            'isResolved': obj.get_status() == STATUS_RESOLVED,
+            'status': status_label,
+            'isResolved': obj.get_status() == GroupStatus.RESOLVED,
             'isPublic': obj.is_public,
             'score': getattr(obj, 'sort_value', 0),
             'project': {
-                'name': obj.project.name,
+                'name': escape(obj.project.name),
                 'slug': obj.project.slug,
             },
-            'version': time.time(),
+            'version': version,
         }
         if hasattr(obj, 'is_bookmarked'):
             d['isBookmarked'] = obj.is_bookmarked
