@@ -5,7 +5,9 @@ sentry.web.frontend.users
 :copyright: (c) 2012 by the Sentry Team, see AUTHORS for more details.
 :license: BSD, see LICENSE for more details.
 """
-from sentry.models import TrackedUser
+from __future__ import absolute_import
+
+from sentry.models import TagValue, Group
 from sentry.web.decorators import login_required, has_access
 from sentry.web.helpers import render_to_response
 
@@ -19,27 +21,25 @@ SORT_OPTIONS = {
 
 @has_access
 @login_required
-def user_list(request, team):
+def user_list(request, organization, project):
     sort = request.GET.get('sort')
     if sort not in SORT_OPTIONS:
         sort = DEFAULT_SORT_OPTION
 
-    # TODO: TrackedUser needs to be team-bound before we can launch it
-    user_list = TrackedUser.objects.filter(project__team=team)
+    tag_list = TagValue.objects.filter(project=project, key='sentry:user')
 
     if sort == 'recent':
-        user_list = user_list.order_by('-last_seen')
+        tag_list = tag_list.order_by('-last_seen')
     elif sort == 'newest':
-        user_list = user_list.order_by('-first_seen')
+        tag_list = tag_list.order_by('-first_seen')
     elif sort == 'events':
-        user_list = user_list.order_by('-num_events')
-
-    # TODO: add separate pain for unbound users (e.g. missing email addresses)
-    user_list = user_list.filter(email__isnull=False)
+        tag_list = tag_list.order_by('-times_seen')
 
     return render_to_response('sentry/users/list.html', {
-        'tuser_list': user_list,
-        'team': team,
+        'team': project.team,
+        'organization': organization,
+        'project': project,
+        'tag_list': tag_list,
         'sort_label': SORT_OPTIONS[sort],
         'SECTION': 'users',
         'SORT_OPTIONS': SORT_OPTIONS,
@@ -48,14 +48,24 @@ def user_list(request, team):
 
 @has_access
 @login_required
-def user_details(request, team, user_id):
-    user = TrackedUser.objects.get(project__team=team, id=user_id)
+def user_details(request, organization, project, user_id):
+    tag = TagValue.objects.get(
+        project=project,
+        key='sentry:user',
+        id=user_id,
+    )
 
-    event_list = user.groups.all()
+    event_list = Group.objects.filter(
+        grouptag__project=project,
+        grouptag__key='sentry:user',
+        grouptag__value=tag.value,
+    )
 
     return render_to_response('sentry/users/details.html', {
-        'team': team,
-        'tuser': user,
+        'team': project.team,
+        'organization': organization,
+        'project': project,
+        'tag': tag,
         'event_list': event_list,
         'SECTION': 'users',
     }, request)
